@@ -1,4 +1,4 @@
-from capa.features import Characteristic
+from capa.features.common import Characteristic
 from capa.features.extractors import loops
 
 
@@ -28,7 +28,7 @@ def extract_features(f):
       f (smda.common.SmdaFunction): the function from which to extract features
 
     yields:
-      Feature, set[VA]: the features and their location found in this function.
+      Tuple[Feature, int]: the features and their location found in this function.
     """
     for func_handler in FUNCTION_HANDLERS:
         for feature, va in func_handler(f):
